/**************************************************************
	PIC melody
	 for the PIC12F1822
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.00
	Tools:
	MPLAB v8.80
	Hitech PIC C pro9.83
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20110830 22:00	1.00	---		Begin the program
20111218 01:00	1.01	---		add "Jingle Bells" of new song data 

---------------------------------------------------------------
port assignment
1	VDD					8	VSS
2	RA5	O LED			7	RA0	(PGD)
3	RA4	(not used)		6	RA1	(PGC)
4	RA3	I SWITCH(VPP)	5	RA2	O	PIEZO_OUT

VDD 2.2V to 5V
RA3 is MUST PULLUP 10K OHM
attention:
Do not RA3 push-switch when Pickit2/ICD2 is active.
Idle current 20uA at 5V

*/

#include <htc.h>
#include "keydefine.h"	//yf[^ keydefine.c

#pragma	jis
#define MHz 000000
#define _XTAL_FREQ 4MHz

__CONFIG(	
        FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF 
        & CPD_OFF & BOREN_NSLEEP & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF
);

__CONFIG(
//	WRT_OFF & PLLEN_OFF & STVREN_ON & BORV_19 &  LVP_OFF
	WRT_OFF & PLLEN_OFF & STVREN_ON & BORV_LO &  LVP_OFF

);

//output port					//	  76543210
#define sound_output 	0x04	//RA2 00000100
#define led_output 		0x20 	//RA5 00100000
#define keyinput 		0x08 	//RA3 00001000
//output port (individual)
#define SoundPort	RA2
#define LEDPort		RA5

//Tempo trim (0 to 4)
#define tempo_multi 8;

//Other define
#define Command_mode	255




//Global settings
unsigned char ToneSet;
unsigned char TempoCount = 100;	//Default TEMPO

//Interrupt service routine
void interrupt isr(void){
	TMR0 = ToneSet;
	if (ToneSet != 0) {
		if (SoundPort) SoundPort = 0 ;
			else SoundPort = 1; //alternate port 
	}
	T0IF =0;
}

//tempo control
void wtime(unsigned char t0){
	unsigned int t1;
	while(t0--){
		t1 = TempoCount * 4;
		while ( --t1 ){
			__delay_us(10);
		}	
	}
}

void sleep_control(void){
	//power up flash
	LEDPort = 0;	//LED light
	__delay_ms(100);
	LEDPort = 1;	//LED off
	GIE  = 0;//Int off
	T0IE = 0;//timer interrupt is off
	while ( (PORTA & keyinput)==0 );//wait if key is pushing
	TRISA = 0b11111111;//PORTA is all input
	NOP();	//Stabilize time
	WPUA  = 0b00110111;//ALL Pin Pull Up(then Low power)
	IOCAN = keyinput;	// checking switch port
	IOCAF = 0;//clear INTERRUPT-ON-CHANGE PORTA FLAG REGISTER
	IOCIF  = 0;//GPIF flag clear
	IOCIE  = 1;//Sleep Wakeup enable
	GIE  = 0;//Interrupt service is disable

	//vector non jump and execute next command
	OSCCON = 0b00000010; // PLL disable, 32KHz internal clock
	SBOREN = 0; //Brown out reset is disable

	SLEEP();//sleep!
	NOP();//Wake up

	// Wakeup when switch is LOW 
	SBOREN = 1; //Brown out reset is enable
	OSCCON = 0b01101010; // PLL disable, 4MHz internal clock
	IOCIE = 0;//PORTA int disable
	IOCIF = 0;//GPIO int flag reset
	IOCAF = 0;//clear INTERRUPT-ON-CHANGE PORTA FLAG REGISTER
	T0IE = 1;//timer on
	PORTA = PORTA;	// Bug avoid
	TRISA = ~(sound_output|led_output);
	GIE	 = 1; //Interrupt is All enable	
	PORTA &= ~led_output;//LED flash
	while ( (PORTA & keyinput)==0 );//wait if key is pushing
}
	
void delay_10ms(unsigned char temp){	//10ms multiple x times ***2.01
	while(temp--){
		__delay_ms(10);
	}	
}
	
		
void main(void){
	OSCCON = 0b01101010; // PLL disable, 4MHz internal clock
	unsigned int n=0;
	unsigned char tmp,tmp2;
 	ANSELA = 0;	//ALL digital port
	PORTA  = 0b00000000;
	TRISA  = ~(sound_output|led_output);//LED and piezo (GP2,GP5)
	TMR0 =0x00 ;
	GIE  = 1;
	PEIE = 1;
	ToneSet =0x00;
	OPTION_REG = 0b00000010; // prescale 8
	T0IE = 1;
	sleep_control();//sleep
	while(1){	
		//command detect
		tmp=songdata[n];
		tmp2=songdata[n+1];
		if (tmp == Command_mode ) {	//Command?
			if (tmp2 == END){	//Tempo or End of data?
				n=0;
				sleep_control();//sleep
			}
			else{
				TempoCount = tmp2;	//New tempo change
				n +=2;
			}
		}
		else{
			if ((PORTA&keyinput)==0) sleep_control();// resume
			ToneSet=tmp;
			wtime(tmp2);
			PORTA ^= led_output;//LED flash
			n +=2;		
		}
	}
}
